@@ -42,7 +42,9 @@
 #include "lib_common/Utils.h"
 #include "lib_common/SPS.h"
 #include "lib_common/PPS.h"
+#include "lib_common/Nuts.h"
 #include "lib_common/ScalingList.h"
+#include "lib_common_enc/PictureInfo.h"
 #include "lib_assert/al_assert.h"
 
 /******************************************************************************/
@@ -613,10 +615,10 @@ static void writePpsData(AL_TBitStreamLite* pBS, AL_THevcPps const* pPps)
     AL_BitStreamLite_PutBit(pBS, 0);
 
     for(iClmn = 0; iClmn < pPps->num_tile_columns_minus1; ++iClmn)
-      AL_BitStreamLite_PutUE(pBS, pPps->column_width[iClmn] - 1);
+      AL_BitStreamLite_PutUE(pBS, pPps->tile_column_width[iClmn] - 1);
 
     for(iRow = 0; iRow < pPps->num_tile_rows_minus1; ++iRow)
-      AL_BitStreamLite_PutUE(pBS, pPps->row_height[iRow] - 1);
+      AL_BitStreamLite_PutUE(pBS, pPps->tile_row_height[iRow] - 1);
 
     AL_BitStreamLite_PutBit(pBS, pPps->loop_filter_across_tiles_enabled_flag);
   }
@@ -815,8 +817,8 @@ static void writeSeiPictureTiming(AL_TBitStreamLite* pBS, AL_TSps const* pISps,
 
   if(pSps->vui_param.hrd_param.nal_hrd_parameters_present_flag || pSps->vui_param.hrd_param.vcl_hrd_parameters_present_flag)
   {
-    AL_BitStreamLite_PutU(pBS, 31, iAuCpbRemovalDelay / 2);
-    AL_BitStreamLite_PutU(pBS, 31, iPicDpbOutputDelay / 2);
+    AL_BitStreamLite_PutU(pBS, pSps->vui_param.hrd_param.au_cpb_removal_delay_length_minus1 + 1, (iAuCpbRemovalDelay / PicStructToFieldNumber[iPicStruct]));
+    AL_BitStreamLite_PutU(pBS, pSps->vui_param.hrd_param.au_cpb_removal_delay_length_minus1 + 1, (iPicDpbOutputDelay / PicStructToFieldNumber[iPicStruct]));
 
     if(pSps->vui_param.hrd_param.sub_pic_hrd_params_present_flag)
     {
@@ -862,11 +864,37 @@ static AL_ECodec getCodec(void)
   return AL_CODEC_HEVC;
 }
 
+static inline bool isForce4BytesCode(AL_EStartCodeBytesAlignedMode eMode, int nut)
+{
+  switch(eMode)
+  {
+  case AL_START_CODE_AUTO: return nut >= AL_HEVC_NUT_VPS && nut <= AL_HEVC_NUT_SUFFIX_SEI;
+  case AL_START_CODE_3_BYTES: return false;
+  case AL_START_CODE_4_BYTES: return true;
+  default: return nut >= AL_HEVC_NUT_VPS && nut <= AL_HEVC_NUT_SUFFIX_SEI;
+  }
+
+  return nut >= AL_HEVC_NUT_VPS && nut <= AL_HEVC_NUT_SUFFIX_SEI;
+}
+
+/******************************************************************************/
+static void writeStartCode(AL_TBitStreamLite* pBS, int nut, AL_EStartCodeBytesAlignedMode eStartCodeBytesAligned)
+{
+  if(isForce4BytesCode(eStartCodeBytesAligned, nut))
+    AL_BitStreamLite_PutBits(pBS, 8, 0x00);
+
+  // don't count start code in case of "VCL Compliance"
+  AL_BitStreamLite_PutBits(pBS, 8, 0x00);
+  AL_BitStreamLite_PutBits(pBS, 8, 0x00);
+  AL_BitStreamLite_PutBits(pBS, 8, 0x01);
+}
+
 /******************************************************************************/
 static IRbspWriter writer =
 {
   getCodec,
   AL_RbspEncoding_WriteAUD,
+  writeStartCode,
   writeVps,
   writeSps,
   writePps,
@@ -876,6 +904,7 @@ static IRbspWriter writer =
   writeSeiPictureTiming,
   AL_RbspEncoding_WriteMasteringDisplayColourVolume,
   AL_RbspEncoding_WriteContentLightLevel,
+  AL_RbspEncoding_WriteAlternativeTransferCharacteristics,
   AL_RbspEncoding_WriteST2094_10,
   AL_RbspEncoding_WriteST2094_40,
   AL_RbspEncoding_WriteUserDataUnregistered
