@@ -106,6 +106,7 @@ static bool AL_sSettings_CheckProfile(AL_EProfile eProfile)
   case AL_PROFILE_HEVC_MAIN_422_12:
   case AL_PROFILE_HEVC_MAIN_444:
   case AL_PROFILE_HEVC_MAIN_444_10:
+  case AL_PROFILE_HEVC_MAIN_444_12:
   case AL_PROFILE_HEVC_MAIN_INTRA:
   case AL_PROFILE_HEVC_MAIN10_INTRA:
   case AL_PROFILE_HEVC_MAIN_422_INTRA:
@@ -368,6 +369,7 @@ static void XAVC_CheckCoherency(AL_TEncSettings* pSettings)
   pChannel->eEncTools &= (~AL_OPT_LF_X_TILE);
   pChannel->eEncTools &= (~AL_OPT_LF_X_SLICE);
   pChannel->eEncTools &= (~AL_OPT_LF);
+  pChannel->bUseUniformSliceType = true;
 
   if(AL_IS_INTRA_PROFILE(pChannel->eProfile))
   {
@@ -438,12 +440,6 @@ static void XAVC_CheckCoherency(AL_TEncSettings* pSettings)
   pGop->uFreqIDR = Min(pGop->uGopLength, pGop->uFreqIDR);
 }
 
-/***************************************************************************/
-static void AL_sSettings_SetDefaultXAVCParam(AL_TEncSettings* pSettings)
-{
-  XAVC_CheckCoherency(pSettings);
-}
-
 /***************************************************************************/
 static void AL_sSettings_SetDefaultAVCParam(AL_TEncSettings* pSettings)
 {
@@ -451,9 +447,6 @@ static void AL_sSettings_SetDefaultAVCParam(AL_TEncSettings* pSettings)
 
   if(pSettings->eScalingList == AL_SCL_MAX_ENUM)
     pSettings->eScalingList = AL_SCL_FLAT;
-
-  if(AL_IS_XAVC(pSettings->tChParam[0].eProfile))
-    AL_sSettings_SetDefaultXAVCParam(pSettings);
 }
 
 /***************************************************************************/
@@ -461,9 +454,6 @@ static void AL_sSettings_SetDefaultHEVCParam(AL_TEncSettings* pSettings)
 {
   if(pSettings->eScalingList == AL_SCL_MAX_ENUM)
     pSettings->eScalingList = AL_SCL_DEFAULT;
-
-  if(pSettings->tChParam[0].uCabacInitIdc > 1)
-    pSettings->tChParam[0].uCabacInitIdc = 1;
 }
 
 /***************************************************************************/
@@ -483,9 +473,9 @@ void AL_Settings_SetDefaultRCParam(AL_TRCParam* pRCParam)
   pRCParam->uPBDelta = -1;
   pRCParam->uMaxPelVal = 255;
   pRCParam->uMaxPSNR = 4200;
-  pRCParam->eOptions = AL_RC_OPT_SCN_CHG_RES | AL_RC_OPT_SC_PREVENTION;
+  pRCParam->eOptions = 0;
 
-  pRCParam->bUseGoldenRef = false;
+  pRCParam->bUseGoldenRef = true;
   pRCParam->uGoldenRefFrequency = 10;
   pRCParam->uPGoldenDelta = 2;
   pRCParam->pMaxPictureSize[AL_SLICE_I] = 0;
@@ -497,6 +487,8 @@ void AL_Settings_SetDefaultRCParam(AL_TRCParam* pRCParam)
 /***************************************************************************/
 #define AL_DEFAULT_PROFILE AL_PROFILE_HEVC_MAIN
 
+static int const iBetaOffsetAuto = -1;
+
 /***************************************************************************/
 void AL_Settings_SetDefaults(AL_TEncSettings* pSettings)
 {
@@ -532,7 +524,7 @@ void AL_Settings_SetDefaults(AL_TEncSettings* pSettings)
   AL_Settings_SetDefaultRCParam(&pChan->tRCParam);
 
   pChan->iTcOffset = -1;
-  pChan->iBetaOffset = -1;
+  pChan->iBetaOffset = iBetaOffsetAuto;
 
   pChan->uNumCore = NUMCORE_AUTO;
   pChan->uNumSlices = 1;
@@ -558,6 +550,7 @@ void AL_Settings_SetDefaults(AL_TEncSettings* pSettings)
   pChan->pMeRange[AL_SLICE_P][1] = -1; // Vert
   pChan->pMeRange[AL_SLICE_B][0] = -1; // Horz
   pChan->pMeRange[AL_SLICE_B][1] = -1; // Vert
+  pChan->uMVVRange = 0;
   pChan->uLog2MaxCuSize = HEVC_MAX_CTB_SIZE;
   pChan->uLog2MinCuSize = MIN_CU_SIZE;
   pChan->uLog2MaxTuSize = 5; // 32x32
@@ -570,7 +563,7 @@ void AL_Settings_SetDefaults(AL_TEncSettings* pSettings)
   pChan->eEntropyMode = AL_MODE_CABAC;
   pChan->eWPMode = AL_WP_DEFAULT;
 
-  pChan->eSrcMode = AL_SRC_NVX;
+  pChan->eSrcMode = AL_SRC_RASTER;
 
   pSettings->LookAhead = 0;
   pSettings->TwoPass = 0;
@@ -584,6 +577,11 @@ void AL_Settings_SetDefaults(AL_TEncSettings* pSettings)
   pChan->uOutputCropPosY = 0;
   pChan->uOutputCropWidth = 0;
   pChan->uOutputCropHeight = 0;
+  pChan->eStartCodeBytesAligned = AL_START_CODE_AUTO;
+
+#if (defined(ANDROID) || defined(__ANDROID_API__))
+  pChan->eStartCodeBytesAligned = AL_START_CODE_4_BYTES;
+#endif
 }
 
 /***************************************************************************/
@@ -608,6 +606,7 @@ void AL_Settings_SetDefaultParam(AL_TEncSettings* pSettings)
 
   if(AL_IS_HEVC(pSettings->tChParam[0].eProfile))
     AL_sSettings_SetDefaultHEVCParam(pSettings);
+
 }
 
 /***************************************************************************/
@@ -621,13 +620,13 @@ int AL_Settings_CheckValidity(AL_TEncSettings* pSettings, AL_TEncChanParam* pChP
     MSG("Invalid parameter: Profile");
   }
 
-  if(AL_IS_10BIT_PROFILE(pChParam->eProfile) && (AL_GET_BITDEPTH(pChParam->ePicFormat) > 8) && (HW_IP_BIT_DEPTH < 10))
+  if(AL_IS_10BIT_PROFILE(pChParam->eProfile) && (AL_GET_BITDEPTH(pChParam->ePicFormat) > 8) && (AL_HWConfig_Enc_GetSupportedBitDepth() < 10))
   {
     ++err;
     MSG("The hardware IP doesn't support 10-bit encoding");
   }
 
-  if(AL_IS_12BIT_PROFILE(pChParam->eProfile) && (AL_GET_BITDEPTH(pChParam->ePicFormat) > 10) && (HW_IP_BIT_DEPTH < 12))
+  if(AL_IS_12BIT_PROFILE(pChParam->eProfile) && (AL_GET_BITDEPTH(pChParam->ePicFormat) > 10) && (AL_HWConfig_Enc_GetSupportedBitDepth() < 12))
   {
     ++err;
     MSG("The hardware IP doesn't support 12-bit encoding");
@@ -641,10 +640,23 @@ int AL_Settings_CheckValidity(AL_TEncSettings* pSettings, AL_TEncChanParam* pChP
     }
   }
 
-  if(pChParam->uSrcWidth != pChParam->uEncWidth || pChParam->uSrcHeight != pChParam->uEncHeight)
+  if(pChParam->bEnableSrcCrop)
   {
-    ++err;
-    MSG("Different input and output resolutions are not supported by the IP");
+    const int HStep = AL_GET_BITDEPTH(pChParam->ePicFormat) == 10 ? 24 : 32; // In 10-bit there are 24 samples every 32 bytes
+    const int VStep = 1; // should be 2 in 4:2:0 but customer requires it to be 1 in any case !
+
+    if((pChParam->uSrcCropPosX % HStep) || (pChParam->uSrcCropPosY % VStep))
+    {
+      ++err;
+      MSG("The input crop area doesn't fit the alignement constraints for the current buffer format");
+    }
+
+    if(pChParam->uSrcCropPosX + pChParam->uSrcCropWidth > pChParam->uSrcWidth ||
+       pChParam->uSrcCropPosY + pChParam->uSrcCropHeight > pChParam->uSrcHeight)
+    {
+      ++err;
+      MSG("The input crop area doesn't fit in the input picture");
+    }
   }
 
   if(!AL_sSettings_CheckLevel(pChParam->eProfile, pChParam->uLevel))
@@ -690,6 +702,12 @@ int AL_Settings_CheckValidity(AL_TEncSettings* pSettings, AL_TEncChanParam* pChP
       MSGF("Invalid parameter: NumCore. The width should at least be %d CTB per core. With the specified number of core, it is %d CTB per core. (Multi core alignement constraint might be the reason of this error if the CTB are equal)", diagnostic.requiredWidthInCtbPerCore, diagnostic.actualWidthInCtbPerCore);
     }
 
+    if(AL_IS_HEVC(pChParam->eProfile) && pSettings->tChParam[0].uCabacInitIdc > 1)
+    {
+      ++err;
+      MSG("Invalid parameter: CabacInit");
+    }
+
     int MinCoreWidth = 256;
 
     if(AL_IS_HEVC(pChParam->eProfile) && (pChParam->uEncWidth < MinCoreWidth * pChParam->uNumCore))
@@ -699,6 +717,11 @@ int AL_Settings_CheckValidity(AL_TEncSettings* pSettings, AL_TEncChanParam* pChP
     }
   }
 
+  if(AL_IS_AVC(pChParam->eProfile) && pSettings->tChParam[0].uCabacInitIdc > 2)
+  {
+    ++err;
+    MSG("Invalid parameter: CabacInit");
+  }
   int const iCTBSize = (1 << pChParam->uLog2MaxCuSize);
 
   if(pChParam->uEncHeight <= iCTBSize)
@@ -708,6 +731,25 @@ int AL_Settings_CheckValidity(AL_TEncSettings* pSettings, AL_TEncChanParam* pChP
   }
 
   int iMaxQP = 51;
+  int iMinQP = 0;
+
+  if(pChParam->tRCParam.iMaxQP < pChParam->tRCParam.iMinQP)
+  {
+    ++err;
+    MSG("Invalid parameters: MinQP and MaxQP");
+  }
+
+  if(pChParam->tRCParam.iMaxQP > iMaxQP)
+  {
+    ++err;
+    MSG("Invalid parameter: MaxQP");
+  }
+
+  if(pChParam->tRCParam.iMinQP < iMinQP)
+  {
+    ++err;
+    MSG("Invalid parameter: MinQP");
+  }
 
   if(pChParam->tRCParam.iInitialQP > iMaxQP)
   {
@@ -790,7 +832,7 @@ int AL_Settings_CheckValidity(AL_TEncSettings* pSettings, AL_TEncChanParam* pChP
   }
 
   AL_EChromaMode eChromaMode = AL_GET_CHROMA_MODE(pChParam->ePicFormat);
-  ECheckResolutionError eResError = AL_ParamConstraints_CheckResolution(pChParam->eProfile, eChromaMode, pChParam->uEncWidth, pChParam->uEncHeight);
+  ECheckResolutionError eResError = AL_ParamConstraints_CheckResolution(pChParam->eProfile, eChromaMode, 1 << pChParam->uLog2MaxCuSize, pChParam->uEncWidth, pChParam->uEncHeight);
 
   if(eResError == CRERROR_WIDTHCHROMA)
   {
@@ -802,6 +844,11 @@ int AL_Settings_CheckValidity(AL_TEncSettings* pSettings, AL_TEncChanParam* pChP
     ++err;
     MSG("Width shall be multiple of 2 on 420 or 422 chroma mode!");
   }
+  else if(eResError == CRERROR_64x64_MIN_RES)
+  {
+    ++err;
+    MSG("LCU64x64 encoding is currently limited to resolution higher or equal to 72x72!");
+  }
 
   int iNumB = pChParam->tGopParam.uNumB;
 
@@ -933,37 +980,6 @@ int AL_Settings_CheckValidity(AL_TEncSettings* pSettings, AL_TEncChanParam* pChP
   return err;
 }
 
-/******************************************************************************/
-static uint32_t GetHevcMaxTileRow(uint8_t uLevel)
-{
-  switch(uLevel)
-  {
-  case 10:
-  case 20:
-  case 21:
-    return 1;
-  case 30:
-    return 2;
-  case 31:
-    return 3;
-  case 40:
-  case 41:
-    return 5;
-  case 50:
-  case 51:
-  case 52:
-    return 11;
-  case 60:
-  case 61:
-  case 62:
-    return 22;
-  default:
-    Rtos_Log(AL_LOG_CRITICAL, "level:%d\n", uLevel);
-    AL_Assert(0);
-    return 1;
-  }
-}
-
 /***************************************************************************/
 bool checkBitDepthCoherency(int iBitDepth, AL_EProfile eProfile)
 {
@@ -1038,7 +1054,7 @@ AL_EProfile getHevcMinimumProfile(int iBitDepth, AL_EChromaMode eChroma)
     // case AL_CHROMA_4_0_0: return AL_PROFILE_HEVC_MONO12;
     case AL_CHROMA_4_2_0: return AL_PROFILE_HEVC_MAIN12;
     case AL_CHROMA_4_2_2: return AL_PROFILE_HEVC_MAIN_422_12;
-    // case AL_CHROMA_4_4_4: return AL_PROFILE_HEVC_MAIN_444_12;
+    case AL_CHROMA_4_4_4: return AL_PROFILE_HEVC_MAIN_444_12;
     default: AL_Assert(0);
     }
 
@@ -1094,6 +1110,7 @@ AL_EProfile getAvcMinimumProfile(int iBitDepth, AL_EChromaMode eChroma)
 AL_EProfile getMinimumProfile(AL_ECodec eCodec, int iBitDepth, AL_EChromaMode eChromaMode)
 {
   (void)iBitDepth;
+  (void)eChromaMode;
   switch(eCodec)
   {
   case AL_CODEC_AVC: return getAvcMinimumProfile(iBitDepth, eChromaMode);
@@ -1125,15 +1142,64 @@ int AL_Settings_CheckCoherency(AL_TEncSettings* pSettings, AL_TEncChanParam* pCh
   if(pChParam->tGopParam.eMode == AL_GOP_MODE_DEFAULT &&
      pChParam->tGopParam.uNumB > uMaxNumB)
   {
-    MSG("!! Warning : number of B frames per GOP is too high");
+    MSG("!! Warning: number of B frames per GOP is too high");
     pChParam->tGopParam.uNumB = uMaxNumB;
   }
 
+  if(!AL_IS_AVC(pChParam->eProfile) && pChParam->bUseUniformSliceType)
+  {
+    MSG("!! Use uniform slice type is not available in this codec !!");
+    pChParam->bUseUniformSliceType = false;
+  }
+
+  if((pChParam->uSliceSize > 0) && (pChParam->uLog2MaxCuSize > 5))
+  {
+    MSG("!! Warning: Slice size is not available with Log2MaxCuSize > 5 !!");
+    pChParam->uSliceSize = 0;
+    ++numIncoherency;
+  }
+
+  bool bIsLoopFilterEnable = (pChParam->eEncTools & AL_OPT_LF);
+
+  if(pChParam->tGopParam.eGdrMode != AL_GDR_OFF)
+  {
+    if((pSettings->uEnableSEI & AL_SEI_RP) == 0)
+    {
+      pSettings->uEnableSEI |= AL_SEI_RP;
+      MSG("!! Force Recovery Point SEI in GDR !!");
+    }
+
+    if(bIsLoopFilterEnable)
+    {
+      pChParam->eEncTools &= ~AL_OPT_LF;
+      MSG("!! Loop filter is not allowed with GDR enabled !!");
+      bIsLoopFilterEnable = false;
+    }
+  }
+
+#if (defined(ANDROID) || defined(__ANDROID_API__))
+
+  if(AL_IS_ITU_CODEC(AL_GET_CODEC(pChParam->eProfile)))
+  {
+    if(pChParam->eStartCodeBytesAligned != AL_START_CODE_4_BYTES)
+    {
+      MSG("!! Start code bytes must be 4-bytes aligned !!");
+      pChParam->eStartCodeBytesAligned = AL_START_CODE_4_BYTES;
+    }
+  }
+#endif
+
+  if(!bIsLoopFilterEnable)
+  {
+    pChParam->iBetaOffset = 0;
+    pChParam->iTcOffset = 0;
+  }
+
   AL_EChromaMode eChromaMode = AL_GET_CHROMA_MODE(pChParam->ePicFormat);
 
   if(!checkProfileCoherency(iBitDepth, eChromaMode, pSettings->tChParam[0].eProfile))
   {
-    MSG("!! Warning : Adapting profile to support bitdepth and chroma mode");
+    MSG("!! Warning: Adapting profile to support bitdepth and chroma mode");
     pSettings->tChParam[0].eProfile = getMinimumProfile(AL_GET_CODEC(pSettings->tChParam[0].eProfile), iBitDepth, eChromaMode);
     ++numIncoherency;
   }
@@ -1142,17 +1208,26 @@ int AL_Settings_CheckCoherency(AL_TEncSettings* pSettings, AL_TEncChanParam* pCh
   {
     if((pChParam->uLog2MaxCuSize - pChParam->uCuQPDeltaDepth) < pChParam->uLog2MinCuSize)
     {
-      MSG("!! Warning : CuQpDeltaDepth doesn't match Log2MinCUSize !!");
+      MSG("!! Warning: CuQpDeltaDepth doesn't match Log2MinCUSize !!");
       ++numIncoherency;
       pChParam->uCuQPDeltaDepth = pChParam->uLog2MaxCuSize - pChParam->uLog2MinCuSize;
     }
 
-    if(pChParam->uCuQPDeltaDepth > 2)
+    if((pChParam->uLog2MaxCuSize == 6 && (pChParam->uCuQPDeltaDepth > 3 || pChParam->uCuQPDeltaDepth > 3)) &&
+       (pChParam->uLog2MaxCuSize != 6 && (pChParam->uCuQPDeltaDepth > 2)))
     {
-      MSG("!! Warning : CuQpDeltaDepth shall be less then or equal to 2 !");
+      MSG("!! Warning: CuQpDeltaDepth is too high!");
       ++numIncoherency;
-      pChParam->uCuQPDeltaDepth = 2;
+      pChParam->uCuQPDeltaDepth = pChParam->uLog2MaxCuSize == 6 ? 3 : 2;
     }
+
+    if(AL_IS_AUTO_OR_ADAPTIVE_QP_CTRL(pSettings->eQpCtrlMode) && pChParam->uLog2MaxCuSize == 6 && pChParam->uCuQPDeltaDepth == 0)
+    {
+      MSG("!! Warning: CuQPDeltaDepth shall be superior to zero in AUTO_QP mode!");
+      ++numIncoherency;
+      pChParam->uCuQPDeltaDepth = 1;
+    }
+
   }
   else
   {
@@ -1302,6 +1377,9 @@ int AL_Settings_CheckCoherency(AL_TEncSettings* pSettings, AL_TEncChanParam* pCh
   int const MAX_LOW_DELAY_B_GOP_LENGTH = 4;
   int const MIN_LOW_DELAY_B_GOP_LENGTH = 1;
 
+  if(pChParam->tRCParam.bUseGoldenRef && (pChParam->tGopParam.uNumB > 0 || pChParam->tRCParam.eRCMode != AL_RC_CONST_QP))
+    pChParam->tRCParam.bUseGoldenRef = false;
+
   if(pChParam->tGopParam.eMode == AL_GOP_MODE_LOW_DELAY_B && (pChParam->tGopParam.uGopLength > MAX_LOW_DELAY_B_GOP_LENGTH ||
                                                               pChParam->tGopParam.uGopLength < MIN_LOW_DELAY_B_GOP_LENGTH))
   {
@@ -1439,9 +1517,9 @@ int AL_Settings_CheckCoherency(AL_TEncSettings* pSettings, AL_TEncChanParam* pCh
     if(iNumCore == NUMCORE_AUTO)
       iNumCore = AL_CoreConstraint_GetExpectedNumberOfCores(&constraint, pChParam->uEncWidth, pChParam->uEncHeight, pChParam->tRCParam.uFrameRate * 1000, pChParam->tRCParam.uClkRatio);
 
-    if(iNumCore > 1 && pChParam->uNumSlices > GetHevcMaxTileRow(pChParam->uLevel))
+    if(iNumCore > 1 && pChParam->uNumSlices > AL_HEVC_GetMaxTileRows(pChParam->uLevel))
     {
-      pChParam->uNumSlices = Clip3(pChParam->uNumSlices, 1, GetHevcMaxTileRow(pChParam->uLevel));
+      pChParam->uNumSlices = Clip3(pChParam->uNumSlices, 1, AL_HEVC_GetMaxTileRows(pChParam->uLevel));
       MSG("!!With this Configuration, this NumSlices cannot be set!!");
       ++numIncoherency;
     }
@@ -1470,9 +1548,12 @@ int AL_Settings_CheckCoherency(AL_TEncSettings* pSettings, AL_TEncChanParam* pCh
   }
 
   if(pChParam->tRCParam.eRCMode == AL_RC_CONST_QP && pChParam->tRCParam.iInitialQP < 0)
+  {
     pChParam->tRCParam.iInitialQP = 30;
 
-  // Fill zero value with surounding non-zero value if any. Warning : don't change the order of if statements below !!
+  }
+
+  // Fill zero value with surounding non-zero value if any. Warning: don't change the order of if statements below !!
   if(pChParam->tRCParam.pMaxPictureSize[AL_SLICE_P] == 0)
     pChParam->tRCParam.pMaxPictureSize[AL_SLICE_P] = pChParam->tRCParam.pMaxPictureSize[AL_SLICE_I];
 
