@@ -49,9 +49,8 @@
 #include "lib_common/Error.h"
 #include "lib_common/FourCC.h"
 #include "lib_common/SyntaxConversion.h"
+#include "lib_common/HardwareConfig.h"
 
-#include "lib_common_dec/DecSliceParam.h"
-#include "lib_common_dec/DecBuffers.h"
 #include "lib_common_dec/RbspParser.h"
 #include "lib_common_dec/DecInfo.h"
 #include "lib_common_dec/Defines_mcu.h"
@@ -132,21 +131,25 @@ static int getMaxBitDepth(AL_TAvcSps const* pSPS)
 }
 
 /*************************************************************************/
-static int getMaxNumberOfSlices(AL_TStreamSettings const* pStreamSettings, AL_TAvcSps const* pSPS)
+static int getMaxNumberOfSlices(AL_TDecCtx const* pCtx, AL_TAvcSps const* pSPS)
 {
-  int maxSlicesCountSupported = AL_AVC_GetMaxNumberOfSlices(AVC_PROFILE_IDC_HIGH_422, 52, 1, 60, INT32_MAX);
-  return maxSlicesCountSupported; /* TODO : fix bad behaviour in firmware to decrease dynamically the number of slices */
-
-  int macroblocksCountInPicture = (pStreamSettings->tDim.iWidth / 16) * (pStreamSettings->tDim.iHeight / 16);
-
+  AL_TStreamSettings const* pStreamSettings = &pCtx->tStreamSettings;
+  int macroblocksCountInPicture = GetBlk16x16(pStreamSettings->tDim);
   int numUnitsInTick = 1, timeScale = 1;
 
-  if(pSPS->vui_parameters_present_flag)
+  if(pSPS->vui_parameters_present_flag && pSPS->vui_param.vui_timing_info_present_flag)
   {
     numUnitsInTick = pSPS->vui_param.vui_num_units_in_tick;
     timeScale = pSPS->vui_param.vui_time_scale;
   }
-  return Min(maxSlicesCountSupported, AL_AVC_GetMaxNumberOfSlices(pStreamSettings->iProfileIdc, pStreamSettings->iLevel, numUnitsInTick, timeScale, macroblocksCountInPicture));
+
+  if(pCtx->bForceFrameRate)
+  {
+    numUnitsInTick = pCtx->pChanParam->uFrameRate * 2;
+    timeScale = pCtx->pChanParam->uClkRatio;
+  }
+
+  return AL_AVC_GetMaxNumberOfSlices(pStreamSettings->eProfile, pStreamSettings->iLevel, numUnitsInTick, timeScale, macroblocksCountInPicture);
 }
 
 /*****************************************************************************/
@@ -154,23 +157,17 @@ static bool isSPSCompatibleWithStreamSettings(AL_TAvcSps const* pSPS, AL_TStream
 {
   int iSPSLumaBitDepth = pSPS->bit_depth_luma_minus8 + 8;
 
-  if(iSPSLumaBitDepth > HW_IP_BIT_DEPTH)
-    return false;
-
   if((pStreamSettings->iBitDepth > 0) && (pStreamSettings->iBitDepth < iSPSLumaBitDepth))
     return false;
 
   int iSPSChromaBitDepth = pSPS->bit_depth_chroma_minus8 + 8;
 
-  if(iSPSChromaBitDepth > HW_IP_BIT_DEPTH)
-    return false;
-
   if((pStreamSettings->iBitDepth > 0) && (pStreamSettings->iBitDepth < iSPSChromaBitDepth))
     return false;
 
   int iSPSMaxBitDepth = getMaxBitDepth(pSPS);
 
-  if(iSPSMaxBitDepth > HW_IP_BIT_DEPTH)
+  if(iSPSMaxBitDepth > AL_HWConfig_Dec_GetSupportedBitDepth())
     return false;
 
   int iSPSLevel = pSPS->constraint_set3_flag ? 9 : pSPS->level_idc; /* We treat constraint set 3 as a level 9 */
@@ -180,37 +177,40 @@ static bool isSPSCompatibleWithStreamSettings(AL_TAvcSps const* pSPS, AL_TStream
 
   AL_EChromaMode eSPSChromaMode = (AL_EChromaMode)pSPS->chroma_format_idc;
 
-  if((pStreamSettings->eChroma != AL_CHROMA_MAX_ENUM) && (pStreamSettings->eChroma < eSPSChromaMode))
+  if(eSPSChromaMode > AL_HWConfig_Dec_GetSupportedChromaMode())
     return false;
 
-  AL_TCropInfo tSPSCropInfo = extractCropInfo(pSPS);
+  if((pStreamSettings->eChroma != AL_CHROMA_MAX_ENUM) && (pStreamSettings->eChroma < eSPSChromaMode))
+  {
+    Rtos_Log(AL_LOG_ERROR, "Invalid chroma-mode, incompatible with initial setting used for allocation.\n");
+    return false;
+  }
 
-  int iSPSCropWidth = tSPSCropInfo.uCropOffsetLeft + tSPSCropInfo.uCropOffsetRight;
   AL_TDimension tSPSDim = extractDimension(pSPS);
 
-  if((pStreamSettings->tDim.iWidth > 0) && (pStreamSettings->tDim.iWidth < (tSPSDim.iWidth - iSPSCropWidth)))
+  if((pStreamSettings->tDim.iWidth > 0) && (pStreamSettings->tDim.iWidth < tSPSDim.iWidth))
   {
-    Rtos_Log(AL_LOG_ERROR, "Invalid Crop\n");
+    Rtos_Log(AL_LOG_ERROR, "Invalid resolution. Width greater than initial setting in decoder.\n");
     return false;
   }
 
-  if((pStreamSettings->tDim.iWidth > 0) && (tSPSDim.iWidth < AL_CORE_AVC_MIN_RES))
+  static const int32_t iMinResolution = AL_CORE_MIN_CU_NB << AL_CORE_AVC_LOG2_MAX_CU_SIZE;
+
+  if(tSPSDim.iWidth < iMinResolution)
   {
-    Rtos_Log(AL_LOG_ERROR, "Invalid resolution : Width too small\n");
+    Rtos_Log(AL_LOG_ERROR, "Invalid resolution : minimum width is 2 CTBs.\n");
     return false;
   }
 
-  int iSPSCropHeight = tSPSCropInfo.uCropOffsetTop + tSPSCropInfo.uCropOffsetBottom;
-
-  if((pStreamSettings->tDim.iHeight > 0) && (pStreamSettings->tDim.iHeight < (tSPSDim.iHeight - iSPSCropHeight)))
+  if((pStreamSettings->tDim.iHeight > 0) && (pStreamSettings->tDim.iHeight < tSPSDim.iHeight))
   {
-    Rtos_Log(AL_LOG_ERROR, "Invalid Crop\n");
+    Rtos_Log(AL_LOG_ERROR, "Invalid resolution. Height greater than initial setting in decoder.\n");
     return false;
   }
 
-  if((pStreamSettings->tDim.iHeight > 0) && (tSPSDim.iHeight < AL_CORE_AVC_MIN_RES))
+  if(tSPSDim.iHeight < iMinResolution)
   {
-    Rtos_Log(AL_LOG_ERROR, "Invalid resolution : Height too small\n");
+    Rtos_Log(AL_LOG_ERROR, "Invalid resolution : minimum height is 2 CTBs.\n");
     return false;
   }
 
@@ -226,28 +226,43 @@ extern int AVC_GetMinOutputBuffersNeeded(int iDpbMaxBuf, int iStack);
 /******************************************************************************/
 static AL_TStreamSettings extractStreamSettings(AL_TAvcSps const* pSPS)
 {
+  uint32_t uFlags = (pSPS->constraint_set0_flag |
+                     (pSPS->constraint_set1_flag << 1) |
+                     (pSPS->constraint_set2_flag << 2) |
+                     (pSPS->constraint_set3_flag << 3) |
+                     (pSPS->constraint_set4_flag << 4) |
+                     (pSPS->constraint_set5_flag << 5));
+
   AL_TStreamSettings tStreamSettings;
   tStreamSettings.tDim = extractDimension(pSPS);
   tStreamSettings.eChroma = (AL_EChromaMode)pSPS->chroma_format_idc;
   tStreamSettings.iBitDepth = getMaxBitDepth(pSPS);
-  AL_Assert(tStreamSettings.iBitDepth <= HW_IP_BIT_DEPTH);
+  AL_Assert(tStreamSettings.iBitDepth <= AL_HWConfig_Dec_GetSupportedBitDepth());
   tStreamSettings.iLevel = pSPS->constraint_set3_flag ? 9 : pSPS->level_idc;
-  tStreamSettings.iProfileIdc = pSPS->profile_idc;
+  tStreamSettings.eProfile = AL_PROFILE_AVC | pSPS->profile_idc | AL_CS_FLAGS(uFlags);
   tStreamSettings.eSequenceMode = AL_SM_PROGRESSIVE;
   return tStreamSettings;
 }
 
 /******************************************************************************/
-static int getConcealedMaxDPBSize(int iLevel, int iWidth, int iHeight, int iSPSMaxRefFrames)
+int AL_AVC_GetMaxDpbBuffers(AL_TStreamSettings const* pSpsSettings)
 {
-  int iDpbMaxBuf = AL_AVC_GetMaxDPBSize(iLevel, iWidth, iHeight);
+  if(AL_IS_INTRA_PROFILE(pSpsSettings->eProfile))
+    return 2;
+
+  return AL_AVC_GetMaxDPBSize(pSpsSettings->iLevel, pSpsSettings->tDim.iWidth, pSpsSettings->tDim.iHeight);
+}
+
+static int getMaxDPBSize(AL_TStreamSettings const* pSpsSettings, int iSPSMaxRefFrames)
+{
+  int iDpbMaxBuf = AL_AVC_GetMaxDpbBuffers(pSpsSettings);
   return Max(iDpbMaxBuf, iSPSMaxRefFrames);
 }
 
 /******************************************************************************/
 static AL_ERR resolutionFound(AL_TDecCtx* pCtx, AL_TStreamSettings const* pSpsSettings, AL_TCropInfo const* pCropInfo, int iSPSMaxRefFrames)
 {
-  int iDpbMaxBuf = getConcealedMaxDPBSize(pSpsSettings->iLevel, pSpsSettings->tDim.iWidth, pSpsSettings->tDim.iHeight, iSPSMaxRefFrames);
+  int iDpbMaxBuf = getMaxDPBSize(pSpsSettings, iSPSMaxRefFrames);
   int iMaxBuf = AVC_GetMinOutputBuffersNeeded(iDpbMaxBuf, pCtx->iStackSize);
   bool bEnableDisplayCompression;
   AL_EFbStorageMode eDisplayStorageMode = AL_Default_Decoder_GetDisplayStorageMode(pCtx, &bEnableDisplayCompression);
@@ -261,18 +276,18 @@ static bool allocateBuffers(AL_TDecCtx* pCtx, AL_TAvcSps const* pSPS)
 {
   int iNumMBs = (pSPS->pic_width_in_mbs_minus1 + 1) * (pSPS->pic_height_in_map_units_minus1 + 1);
   AL_Assert(iNumMBs == ((pCtx->tStreamSettings.tDim.iWidth / 16) * (pCtx->tStreamSettings.tDim.iHeight / 16)));
-  int iSPSMaxSlices = getMaxNumberOfSlices(&pCtx->tStreamSettings, pSPS);
+  int iSPSMaxSlices = getMaxNumberOfSlices(pCtx, pSPS);
   int iSizeWP = iSPSMaxSlices * WP_SLICE_SIZE;
   int iSizeSP = iSPSMaxSlices * sizeof(AL_TDecSliceParam);
   int iSizeCompData = AL_GetAllocSize_AvcCompData(pCtx->tStreamSettings.tDim, pCtx->tStreamSettings.eChroma);
   int iSizeCompMap = AL_GetAllocSize_DecCompMap(pCtx->tStreamSettings.tDim);
   AL_ERR error = AL_ERR_NO_MEMORY;
 
-  if(!AL_Default_Decoder_AllocPool(pCtx, 0, iSizeWP, iSizeSP, iSizeCompData, iSizeCompMap))
+  if(!AL_Default_Decoder_AllocPool(pCtx, 0, 0, iSizeWP, iSizeSP, iSizeCompData, iSizeCompMap, 0))
     goto fail_alloc;
 
-  int iDpbMaxBuf = getConcealedMaxDPBSize(pCtx->tStreamSettings.iLevel, pCtx->tStreamSettings.tDim.iWidth, pCtx->tStreamSettings.tDim.iHeight, pSPS->max_num_ref_frames);
-  int iMaxBuf = AVC_GetMinOutputBuffersNeeded(iDpbMaxBuf, pCtx->iStackSize);
+  int iDpbMaxBuf = getMaxDPBSize(&pCtx->tStreamSettings, pSPS->max_num_ref_frames);
+  int iMaxBuf = AL_AVC_GetMinOutputBuffersNeeded(&pCtx->tStreamSettings, pCtx->iStackSize);
   int iSizeMV = AL_GetAllocSize_AvcMV(pCtx->tStreamSettings.tDim);
   int iSizePOC = POCBUFF_PL_SIZE;
 
@@ -288,10 +303,10 @@ static bool allocateBuffers(AL_TDecCtx* pCtx, AL_TAvcSps const* pSPS)
     iMaxBuf,
     iSizeMV,
     pCtx->pChanParam->bUseEarlyCallback,
-
+    pCtx->tOutputPosition,
   };
 
-  AL_PictMngr_Init(&pCtx->PictMngr, pCtx->pAllocator, &tPictMngrParam);
+  AL_PictMngr_Init(&pCtx->PictMngr, &tPictMngrParam);
 
   AL_TCropInfo tCropInfo = extractCropInfo(pSPS);
   error = resolutionFound(pCtx, &pCtx->tStreamSettings, &tCropInfo, pSPS->max_num_ref_frames);
@@ -312,11 +327,13 @@ static bool initChannel(AL_TDecCtx* pCtx, AL_TAvcSps const* pSPS)
   AL_TDecChanParam* pChan = pCtx->pChanParam;
   pChan->iWidth = pCtx->tStreamSettings.tDim.iWidth;
   pChan->iHeight = pCtx->tStreamSettings.tDim.iHeight;
+  pChan->uLog2MaxCuSize = AL_CORE_AVC_LOG2_MAX_CU_SIZE;
 
-  const int iSPSMaxSlices = getMaxNumberOfSlices(&pCtx->tStreamSettings, pSPS);
+  int const iSPSMaxSlices = getMaxNumberOfSlices(pCtx, pSPS);
   pChan->iMaxSlices = iSPSMaxSlices;
+  pChan->iMaxTiles = 1;
 
-  if(!pCtx->bForceFrameRate && pSPS->vui_parameters_present_flag)
+  if(!pCtx->bForceFrameRate && pSPS->vui_parameters_present_flag && pSPS->vui_param.vui_timing_info_present_flag)
   {
     pChan->uFrameRate = pSPS->vui_param.vui_time_scale / 2;
     pChan->uClkRatio = pSPS->vui_param.vui_num_units_in_tick;
@@ -457,9 +474,9 @@ static void createConcealSlice(AL_TDecCtx* pCtx, AL_TDecPicParam* pPP, AL_TDecSl
   concealSlice(pCtx, pPP, pSP, pSlice, false);
   pSP->FirstLcuSliceSegment = 0;
   pSP->FirstLcuSlice = 0;
-  pSP->FirstLCU = 0;
+  pSP->SliceFirstLCU = 0;
   pSP->NextSliceSegment = pSlice->first_mb_in_slice;
-  pSP->NumLCU = pSlice->first_mb_in_slice;
+  pSP->SliceNumLCU = pSlice->first_mb_in_slice;
 
   pSlice->slice_type = uCurSliceType;
 }
@@ -597,7 +614,7 @@ static bool avcInitFrameBuffers(AL_TDecCtx* pCtx, bool bStartsNewCVS, const AL_T
   (void)bStartsNewCVS;
   AL_TDimension const tDim = extractDimension(pSPS);
 
-  if(!AL_InitFrameBuffers(pCtx, pBufs, bStartsNewCVS, tDim, pPP))
+  if(!AL_InitFrameBuffers(pCtx, pBufs, bStartsNewCVS, tDim, (AL_EChromaMode)pSPS->chroma_format_idc, pPP))
     return false;
 
   AL_TBuffer* pDispBuf = AL_PictMngr_GetDisplayBufferFromID(&pCtx->PictMngr, pPP->tBufIDs.FrmID);
@@ -607,8 +624,8 @@ static bool avcInitFrameBuffers(AL_TDecCtx* pCtx, bool bStartsNewCVS, const AL_T
   {
     AL_THDRSEIs* pHDRSEIs = &pCtx->aup.tParsedHDRSEIs;
     pMeta->eColourDescription = AL_H273_ColourPrimariesToColourDesc(pSPS->vui_param.colour_primaries);
-    pMeta->eTransferCharacteristics = pSPS->vui_param.transfer_characteristics;
-    pMeta->eColourMatrixCoeffs = pSPS->vui_param.matrix_coefficients;
+    pMeta->eTransferCharacteristics = AL_VUIValueToTransferCharacteristics(pSPS->vui_param.transfer_characteristics);
+    pMeta->eColourMatrixCoeffs = AL_VUIValueToColourMatrixCoefficients(pSPS->vui_param.matrix_coefficients);
     AL_HDRSEIs_Copy(pHDRSEIs, &pMeta->tHDRSEIs);
     AL_HDRSEIs_Reset(pHDRSEIs);
   }
@@ -654,6 +671,9 @@ static void decodeSliceData(AL_TAup* pIAUP, AL_TDecCtx* pCtx, AL_ENut eNUT, bool
 
     if(!initSlice(pCtx, pSlice))
     {
+      if(bIsLastAUNal && *bBeginFrameIsValid)
+        AL_CancelFrameBuffers(pCtx);
+
       UpdateContextAtEndOfFrame(pCtx);
       return;
     }
@@ -688,7 +708,15 @@ static void decodeSliceData(AL_TAup* pIAUP, AL_TDecCtx* pCtx, AL_ENut eNUT, bool
     else if(bCheckDynResChange && (spsSettings.tDim.iWidth != tLastDim.iWidth || spsSettings.tDim.iHeight != tLastDim.iHeight))
     {
       AL_TCropInfo tCropInfo = extractCropInfo(pSPS);
-      resolutionFound(pCtx, &spsSettings, &tCropInfo, pSPS->max_num_ref_frames);
+      AL_ERR error = resolutionFound(pCtx, &spsSettings, &tCropInfo, pSPS->max_num_ref_frames);
+
+      if(error != AL_SUCCESS)
+      {
+        Rtos_Log(AL_LOG_ERROR, "ResolutionFound callback returns with error\n");
+        AL_Default_Decoder_SetError(pCtx, AL_WARN_RES_FOUND_CB, pPP->tBufIDs.FrmID);
+        pSPS->bConceal = true;
+        isValid = false;
+      }
     }
   }
 
